package unversioned

import (
	api "github.com/openshift/origin/pkg/project/api"
	pkg_api "k8s.io/kubernetes/pkg/api"
	watch "k8s.io/kubernetes/pkg/watch"
)

// ProjectsGetter has a method to return a ProjectInterface.
// A group's client should implement this interface.
type ProjectsGetter interface {
	Projects(namespace string) ProjectInterface
}

// ProjectInterface has methods to work with Project resources.
type ProjectInterface interface {
	Create(*api.Project) (*api.Project, error)
	Update(*api.Project) (*api.Project, error)
	Delete(name string, options *pkg_api.DeleteOptions) error
	DeleteCollection(options *pkg_api.DeleteOptions, listOptions pkg_api.ListOptions) error
	Get(name string) (*api.Project, error)
	List(opts pkg_api.ListOptions) (*api.ProjectList, error)
	Watch(opts pkg_api.ListOptions) (watch.Interface, error)
	ProjectExpansion
}

// projects implements ProjectInterface
type projects struct {
	client *CoreClient
	ns     string
}

// newProjects returns a Projects
func newProjects(c *CoreClient, namespace string) *projects {
	return &projects{
		client: c,
		ns:     namespace,
	}
}

// Create takes the representation of a project and creates it.  Returns the server's representation of the project, and an error, if there is any.
func (c *projects) Create(project *api.Project) (result *api.Project, err error) {
	result = &api.Project{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("projects").
		Body(project).
		Do().
		Into(result)
	return
}

// Update takes the representation of a project and updates it. Returns the server's representation of the project, and an error, if there is any.
func (c *projects) Update(project *api.Project) (result *api.Project, err error) {
	result = &api.Project{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("projects").
		Name(project.Name).
		Body(project).
		Do().
		Into(result)
	return
}

// Delete takes name of the project and deletes it. Returns an error if one occurs.
func (c *projects) Delete(name string, options *pkg_api.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("projects").
		Name(name).
		Body(options).
		Do().
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *projects) DeleteCollection(options *pkg_api.DeleteOptions, listOptions pkg_api.ListOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("projects").
		VersionedParams(&listOptions, pkg_api.ParameterCodec).
		Body(options).
		Do().
		Error()
}

// Get takes name of the project, and returns the corresponding project object, and an error if there is any.
func (c *projects) Get(name string) (result *api.Project, err error) {
	result = &api.Project{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("projects").
		Name(name).
		Do().
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of Projects that match those selectors.
func (c *projects) List(opts pkg_api.ListOptions) (result *api.ProjectList, err error) {
	result = &api.ProjectList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("projects").
		VersionedParams(&opts, pkg_api.ParameterCodec).
		Do().
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested projects.
func (c *projects) Watch(opts pkg_api.ListOptions) (watch.Interface, error) {
	return c.client.Get().
		Prefix("watch").
		Namespace(c.ns).
		Resource("projects").
		VersionedParams(&opts, pkg_api.ParameterCodec).
		Watch()
}
